from collections import namedtuple

from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from ..utils import currency


class PaymentStatus(namedtuple('_PaymentsStatus', ('price', 'discount', 'paid'))):

    @property
    def receivable(self):
        return self.price - self.discount

    @property
    def balance(self):
        return self.paid - self.receivable

    @property
    def color(self):
        if self.balance == 0:
            return settings.LEPRIKON_COLOR_PAID
        elif self.balance < 0:
            return settings.LEPRIKON_COLOR_NOTPAID
        else:
            return settings.LEPRIKON_COLOR_OVERPAID

    @property
    def title(self):
        if self.balance == 0:
            return _('paid')
        elif self.balance < 0:
            return _('{} let to pay').format(currency(-self.balance))
        else:
            return _('{} overpaid').format(currency(self.balance))

    def __repr__(self):
        return 'PaymentStatus(price={price}, discount={discount}, paid={paid}, balance={balance})'.format(
            price=self.price,
            discount=self.discount,
            paid=self.paid,
            balance=self.balance,
        )

    def __add__(self, other):
        if other == 0:
            return self
        return PaymentStatus(
            price=self.price + other.price,
            discount=self.discount + other.discount,
            paid=self.paid + other.paid,
        )

    __radd__ = __add__


class BankAccount:
    def __init__(self, iban):
        self.iban = iban

    @cached_property
    def country_code(self):
        return self.iban[:2]

    @cached_property
    def bank_code(self):
        return self.iban[4:8]

    @cached_property
    def account_prefix(self):
        return self.iban[8:14].lstrip('0')

    @cached_property
    def account_number(self):
        return self.iban[14:].lstrip('0')

    def __str__(self):
        return '%s%s%s/%s' % (
            self.account_prefix,
            self.account_prefix and '-',
            self.account_number,
            self.bank_code,
        )


def generate_variable_symbol(registration):
    # get base variable symol from the configured expression
    variable_symbol = eval(settings.LEPRIKON_VARIABLE_SYMBOL_EXPRESSION, {'reg': registration})

    # add check digit
    odd_sum = 0
    even_sum = 0
    for i, char in enumerate(str(variable_symbol)):
        if i % 2:
            even_sum += int(char)
        else:
            odd_sum += int(char)
    check_digit = (odd_sum * 3 + even_sum) % 10
    return variable_symbol * 10 + check_digit



from django.core.urlresolvers import reverse_lazy as reverse
from django.utils.translation import ugettext_lazy as _

from ..forms.parent import ParentForm
from ..models.roles import Parent
from .generic import CreateView, UpdateView


class ParentCreateView(CreateView):
    model = Parent
    form_class = ParentForm
    success_url = reverse('leprikon:summary')
    title = _('New parent')

    def get_form_kwargs(self):
        kwargs = super(ParentCreateView, self).get_form_kwargs()
        kwargs['user'] = self.request.user
        if self.request.user.leprikon_parents.count() == 0:
            kwargs['initial'] = dict(
                (attr, getattr(self.request.user, attr))
                for attr in ['first_name', 'last_name', 'email']
            )
        return kwargs

    def get_message(self):
        return _('New parent {} has been created.').format(self.object)


class ParentUpdateView(UpdateView):
    model = Parent
    form_class = ParentForm
    success_url = reverse('leprikon:summary')
    title = _('Change parent')

    def get_queryset(self):
        # only allow to edit user's own parents
        return super(ParentUpdateView, self).get_queryset().filter(user=self.request.user)

    def get_form_kwargs(self):
        kwargs = super(ParentUpdateView, self).get_form_kwargs()
        kwargs['user'] = self.request.user
        return kwargs

    def get_message(self):
        return _('The parent {} has been updated.').format(self.object)



from django.conf.urls import include, url
from django.db import transaction
from django.views.generic.base import RedirectView

from . import views
from .api import urls as api_urls
from .conf import settings

# dictionary of all LEPRIKON_URL_* attributes of settings
LEPRIKON_URL = dict(
    (attr.lower()[len('LEPRIKON_URL_'):], getattr(settings, attr))
    for attr in dir(settings) if attr.startswith('LEPRIKON_URL_')
)


def d_url(pattern, name):
    return url(pattern.format(**LEPRIKON_URL), transaction.atomic(getattr(views, name)), name=name)


app_name = 'leprikon'

urlpatterns = [
    url(r'^$', RedirectView.as_view(url='{summary}/'.format(**LEPRIKON_URL), permanent=True), name='index'),
    d_url(r'^{summary}/$', 'summary'),

    d_url(r'^{create_account}/$', 'user_create'),
    d_url(r'^{user}/{agreement}/$', 'user_agreement'),
    d_url(r'^{user}/{email}/$', 'user_email'),
    d_url(r'^{user}/$', 'user_update'),
    d_url(r'^{user}/{password}/$', 'user_password'),
    d_url(r'^{login}/$', 'user_login'),
    d_url(r'^{logout}/$', 'user_logout'),
    d_url(r'^{password_reset}/$', 'password_reset'),
    d_url(r'^{password_reset}/done/$', 'password_reset_done'),
    d_url(r'^{password_reset}/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{{1,13}}-[0-9A-Za-z]{{1,20}})/$',
          'password_reset_confirm'),
    d_url(r'^{password_reset}/complete/$', 'password_reset_complete'),

    d_url(r'^{leader}/$', 'leader_summary'),
    d_url(r'^{leader}/{journal}/(?P<pk>[0-9]+)/$', 'journal'),
    d_url(r'^{leader}/{journal}/{journalentry}/{add}/(?P<subject>[0-9]+)/$', 'journalentry_create'),
    d_url(r'^{leader}/{journal}/{journalentry}/(?P<pk>[0-9]+)/$', 'journalentry_update'),
    d_url(r'^{leader}/{journal}/{journalentry}/(?P<pk>[0-9]+)/{delete}/$', 'journalentry_delete'),
    d_url(r'^{leader}/{journal}/{alternating}/$', 'alternating'),
    d_url(r'^{leader}/{timesheets}/$', 'timesheet_list'),
    d_url(r'^{leader}/{timesheets}/(?P<pk>[0-9]+)/$', 'timesheet_detail'),
    d_url(r'^{leader}/{timesheets}/(?P<pk>[0-9]+)/{submit}/$', 'timesheet_submit'),
    d_url(r'^{leader}/{timesheets}/{entry}/{add}/$', 'timesheetentry_create'),
    d_url(r'^{leader}/{timesheets}/{entry}/(?P<pk>[0-9]+)/$', 'timesheetentry_update'),
    d_url(r'^{leader}/{timesheets}/{entry}/(?P<pk>[0-9]+)/{delete}/$', 'timesheetentry_delete'),
    d_url(r'^{leader}/{timesheets}/{journalentry}/(?P<pk>[0-9]+)/$', 'journalleaderentry_update'),
    d_url(r'^{leader}/{timesheets}/{journalentry}/(?P<pk>[0-9]+)/{delete}/$', 'journalleaderentry_delete'),
    d_url(r'^{leader}/(?P<subject_type>[^/]+)/$', 'subject_list_mine'),
    d_url(r'^{leader}/(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{edit}/$', 'subject_update'),
    d_url(r'^{leader}/(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{registrations}/$', 'subject_registrations'),

    d_url(r'^{registrations}/$', 'registration_list'),
    d_url(r'^{registrations}/(?P<pk>[^/]+)/(?P<slug>[^.]+).pdf$', 'registration_pdf'),
    d_url(r'^{registrations}/(?P<pk>[^/]+)/{cancel}/$', 'registration_cancel'),

    d_url(r'^{payments}/$', 'payment_list'),
    d_url(r'^{payments}/(?P<pk>[^/]+)/(?P<slug>[^.]+).pdf$', 'payment_pdf'),

    d_url(r'^{participants}/$', 'participant_list'),
    d_url(r'^{participants}/{add}/$', 'participant_create'),
    d_url(r'^{participants}/(?P<pk>[0-9]+)/$', 'participant_update'),
    d_url(r'^{participants}/{parent}/{add}/$', 'parent_create'),
    d_url(r'^{participants}/{parent}/(?P<pk>[0-9]+)/$', 'parent_update'),

    d_url(r'^{messages}/$', 'message_list'),
    d_url(r'^{messages}/(?P<slug>[^.]+)/$', 'message_detail'),

    d_url(r'^{leaders}/$', 'leader_list'),

    d_url(r'^{school_year}/$', 'school_year'),

    d_url(r'^{reports}/$', 'report_list'),
    d_url(r'^{reports}/{courses}/{payments}/$', 'report_course_payments'),
    d_url(r'^{reports}/{courses}/{payments_status}/$', 'report_course_payments_status'),
    d_url(r'^{reports}/{courses}/{stats}/$', 'report_course_stats'),
    d_url(r'^{reports}/{events}/{payments}/$', 'report_event_payments'),
    d_url(r'^{reports}/{events}/{payments_status}/$', 'report_event_payments_status'),
    d_url(r'^{reports}/{events}/{stats}/$', 'report_event_stats'),
    d_url(r'^{reports}/{debtors}/$', 'report_debtors'),

    d_url(r'^{terms_conditions}/$', 'terms_conditions'),

    url(r'^api/', include(api_urls, 'api')),
]



from django import forms
from django.conf.urls import url
from django.contrib import admin
from django.core.urlresolvers import reverse
from django.db import transaction
from django.http import (
    HttpResponse, HttpResponseBadRequest, HttpResponseRedirect,
)
from django.shortcuts import get_object_or_404, render
from django.utils.translation import ugettext_lazy as _

from ..forms.messages import MessageAdminForm
from ..models.messages import Message, MessageAttachment, MessageRecipient


def messagerecipient_send_mails(request, message, recipients, media):
    return render(
        request,
        'admin/leprikon/messagerecipient/send_mails.html',
        {
            'title': _('Sending emails'),
            'message': message,
            'recipients': recipients,
            'media': media,
            'message_opts': Message._meta,
            'opts': MessageRecipient._meta,
        },
    )


class SendMessageAdminMixin(object):
    actions = ('send_message', 'add_to_message')

    def get_message_recipients(self, request, queryset):
        raise NotImplementedError('{} must implement method get_message_recipients'.format(self.__class__.__name__))

    def send_message(self, request, queryset):
        request.method = 'GET'
        request.leprikon_message_recipients = [r.id for r in self.get_message_recipients(request, queryset)]
        return admin.site._registry[Message].changeform_view(request, form_url=reverse('admin:leprikon_message_add'))
    send_message.short_description = _('Send message')

    def add_to_message(self, request, queryset):

        class MessageForm(forms.Form):
            message = forms.ModelChoiceField(
                label=_('Target message'),
                help_text=_('Recipients will be added to selected message.'),
                queryset=Message.objects.all(),
            )
        if request.POST.get('post', 'no') == 'yes':
            form = MessageForm(request.POST)
            if form.is_valid():
                message = form.cleaned_data['message']
                for recipient in self.get_message_recipients(request, queryset):
                    MessageRecipient.objects.get_or_create(message=message, recipient=recipient)
                return HttpResponseRedirect(
                    reverse('admin:leprikon_messagerecipient_changelist') + '?message={}'.format(message.id)
                )
        else:
            form = MessageForm()
        return render(
            request,
            'admin/leprikon/message/add_recipients.html',
            {
                'title': _('Select target message'),
                'queryset': queryset,
                'opts': self.model._meta,
                'form': form,
                'action_checkbox_name': admin.helpers.ACTION_CHECKBOX_NAME,
            },
        )
    add_to_message.short_description = _('Add recipients to existing message')


class MessageAttachmentInlineAdmin(admin.TabularInline):
    model = MessageAttachment
    extra = 0


@admin.register(Message)
class MessageAdmin(admin.ModelAdmin):
    form = MessageAdminForm
    inlines = (MessageAttachmentInlineAdmin,)
    search_fields = ('subject', 'text')
    list_display = ('subject', 'created', 'recipients', 'action_links')

    def get_changeform_initial_data(self, request):
        initial = super(MessageAdmin, self).get_changeform_initial_data(request)
        try:
            initial['recipients'] = request.leprikon_message_recipients
        except Exception:
            pass
        return initial

    def recipients(self, obj):
        return (
            '<a href="{recipients_url}">'
            '    <span title="{all_title}">{all_count}</span>'
            '  / <span title="{mails_title}">{mails_count}</span>'
            '  / <span title="{viewed_title}">{viewed_count}</span>'
            '</a> '
        ).format(
            recipients_url=reverse('admin:leprikon_messagerecipient_changelist') + '?message={}'.format(obj.id),
            all_title=_('recipients count'),
            mails_title=_('sent mail'),
            viewed_title=_('viewed on site'),
            all_count=obj.recipients.count(),
            mails_count=obj.recipients.exclude(sent_mail=None).count(),
            viewed_count=obj.recipients.exclude(viewed=None).count(),
        )
    recipients.allow_tags = True
    recipients.short_description = _('recipients')

    def action_links(self, obj):
        return (
            '<a href="{recipients_url}" class="button" title="{recipients_title}">{recipients}</a> '
            '<a href="{send_mails_all_url}" class="button" title="{send_mails_all_title}">{send_mails_all}</a> '
            '<a href="{send_mails_new_url}" class="button" title="{send_mails_new_title}">{send_mails_new}</a> '
        ).format(
            recipients=_('recipients'),
            recipients_title=_('show recipients details'),
            recipients_url=(reverse('admin:leprikon_messagerecipient_changelist') + '?message={}'.format(obj.id)),
            send_mails_all=_('send mails'),
            send_mails_all_title=_('send mail to all recipients'),
            send_mails_all_url=reverse('admin:leprikon_message_send_mails') + '?message={}'.format(obj.id),
            send_mails_new=_('send unsent'),
            send_mails_new_title=_('send mails to new recipients only'),
            send_mails_new_url=reverse('admin:leprikon_message_send_mails') + '?message={}&new=1'.format(obj.id),
        )
    action_links.allow_tags = True
    action_links.short_description = _('actions')

    def get_urls(self):
        return [
            url(r'^send-mails/$', self.admin_site.admin_view(self.send_mails), name='leprikon_message_send_mails'),
        ] + super(MessageAdmin, self).get_urls()

    def send_mails(self, request):
        try:
            message_id = int(request.GET['message'])
        except (KeyError, ValueError):
            return HttpResponseBadRequest()
        message = get_object_or_404(Message, id=message_id)
        recipients = message.recipients.all()
        if request.GET.get('new', False):
            recipients = recipients.filter(sent_mail=None)
        return messagerecipient_send_mails(request, message, recipients, self.media)


@admin.register(MessageRecipient)
class MessageRecipientAdmin(admin.ModelAdmin):
    list_display = ('recipient', 'sent', 'viewed', 'sent_mail')
    actions = ('send_mails',)

    # hide the model from admin index
    def get_model_perms(self, request):
        return {}

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def send_mails(self, request, queryset):
        return messagerecipient_send_mails(
            request,
            message=get_object_or_404(Message, id=request.GET.get('message', 0)),
            recipients=queryset,
            media=self.media,
        )
    send_mails.short_description = _('Send email to selected recipients')

    def changelist_view(self, request, extra_context=None):
        message = get_object_or_404(Message, id=request.GET.get('message', 0))
        if extra_context is None:
            extra_context = {}
        extra_context.update({
            'title': _('Recipients of message {}').format(message.subject),
            'message': message,
            'message_opts': Message._meta,
        })
        return super(MessageRecipientAdmin, self).changelist_view(request, extra_context)

    def get_urls(self):
        return [url(
            r'^send-mail/$',
            self.admin_site.admin_view(self.send_mail),
            name='leprikon_messagerecipient_send_mail',
        )] + super(MessageRecipientAdmin, self).get_urls()

    @transaction.atomic
    def send_mail(self, request):
        try:
            recipient_id = int(request.GET['recipient_id'])
        except (KeyError, ValueError):
            return HttpResponseBadRequest()
        recipient = get_object_or_404(MessageRecipient, id=recipient_id)
        recipient.send_mail()
        return HttpResponse('0', content_type="text/json")



import csv
from datetime import datetime
from functools import partial

import django_excel
from django.core.exceptions import FieldDoesNotExist
from django.db.models import F
from django.http import HttpResponse
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _


def get_attr_value(obj, name):
    value = getattr(obj, name)
    if callable(value):
        value = value()
    return value


def get_verbose_name(field):
    try:
        return force_text(field.verbose_name)
    except AttributeError:
        return force_text(field.related_model._meta.verbose_name)


class AdminExportMixin:
    actions = ('export_as_xlsx', 'export_as_csv')

    def get_list_export(self, request):
        try:
            return self.list_export
        except AttributeError:
            return self.list_display

    def get_export_fields(self, request):
        fields = []
        for name in self.get_list_export(request):
            try:
                names = name.split('__')
                field = self.model._meta.get_field(names[0])
                verbose_names = [get_verbose_name(field)]
                for n in names[1:]:
                    field = field.related_model._meta.get_field(n)
                    verbose_names.append(get_verbose_name(field))
                fields.append({
                    'annotate': name if len(names) > 1 else None,
                    'verbose_name': ' / '.join(verbose_names),
                    'get_value': partial(lambda name, obj: get_attr_value(obj, name), name),
                })
            except (AttributeError, FieldDoesNotExist):
                if callable(name):
                    fields.append({
                        'verbose_name': getattr(name, 'short_description', name.__func__.__name__),
                        'get_value': partial(lambda name, obj: name(obj), name),
                    })
                elif hasattr(self, name):
                    attr = getattr(self, name)
                    fields.append({
                        'verbose_name': getattr(attr, 'short_description', name),
                        'get_value': partial(lambda attr, obj: attr(obj), attr),
                    })
                elif hasattr(self.model, name):
                    attr = getattr(self.model, name)
                    fields.append({
                        'verbose_name': getattr(attr, 'short_description', name),
                        'get_value': partial(lambda name, obj: get_attr_value(obj, name), name),
                    })
                else:
                    raise Exception('Can not resolve name "{}"'.format(name))
        return fields

    def get_export_data(self, request, queryset):
        fields = self.get_export_fields(request)
        yield [force_text(f['verbose_name']) for f in fields]
        annotations = {
            field['annotate']: F(field['annotate'])
            for field in fields if field.get('annotate')
        }
        for obj in queryset.annotate(**annotations):
            values = []
            for field in fields:
                value = field['get_value'](obj)
                if value is None:
                    value = ''
                if isinstance(value, datetime):
                    value = value.replace(tzinfo=None)
                else:
                    # what can not be converted to float, must be converted to string
                    try:
                        float(value)
                    except (TypeError, ValueError):
                        value = force_text(value)
                values.append(value)
            yield values

    def export_as_csv(self, request, queryset):
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="{}.csv"'.format(self.model._meta.model_name)
        data = self.get_export_data(request, queryset)
        # write data to response (use all to evaluate the map generator)
        csv.writer(response).writerows(data)
        return response
    export_as_csv.short_description = _('Export selected records as CSV')

    def export_as_xlsx(self, request, queryset):
        data = self.get_export_data(request, queryset)
        response = django_excel.make_response(django_excel.pe.Sheet(data), 'xlsx')
        response['Content-Disposition'] = 'attachment; filename="{}.xlsx"'.format(self.model._meta.model_name)
        return response
    export_as_xlsx.short_description = _('Export selected records as XLSX')



import locale

import icu
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.http import Http404
from django.template.response import TemplateResponse
from django.utils.translation import ugettext_lazy as _

from . import (  # NOQA
    account, agegroup, agreements, citizenship, courses, department, events,
    journals, leprikonsite, messages, place, printsetup, question, roles,
    school, schoolyear, subjects, targetgroup, timesheets, user,
)

User = get_user_model()

admin.site.site_header = _('Leprikon administration')
admin.site.site_title = _('Leprikon administration')


def app_index(self, request, app_label, extra_context=None):
    app_dict = self._build_app_dict(request, app_label)
    if not app_dict:
        raise Http404('The requested admin page does not exist.')
    # Sort the models alphabetically within each app.
    collator = icu.Collator.createInstance(icu.Locale('.'.join(locale.getlocale())))
    app_dict['models'].sort(key=lambda x: collator.getSortKey(x['name'].lower()))
    context = dict(
        self.each_context(request),
        title=_('Leprikon administration'),
        app_list=[app_dict],
        app_label=app_label,
    )
    context.update(extra_context or {})

    request.current_app = self.name

    return TemplateResponse(request, self.app_index_template or [
        'admin/%s/app_index.html' % app_label,
        'admin/app_index.html'
    ], context)


def get_app_list(self, request):
    """
    Returns a sorted list of all the installed apps that have been
    registered in this site.
    """
    collator = icu.Collator.createInstance(icu.Locale('.'.join(locale.getlocale())))

    app_dict = self._build_app_dict(request)

    # Sort the apps alphabetically.
    app_list = sorted(app_dict.values(), key=lambda x: collator.getSortKey(x['name'].lower()))

    # Sort the models alphabetically within each app.
    for app in app_list:
        app['models'].sort(key=lambda x: collator.getSortKey(x['name'].lower()))

    return app_list


# override default Admin site's app_index and get_app_list
admin.sites.AdminSite.app_index = app_index
admin.sites.AdminSite.get_app_list = get_app_list



from collections import namedtuple

from django.core.urlresolvers import reverse_lazy as reverse
from django.db.models import Sum
from django.template.response import TemplateResponse
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from ...forms.reports.courses import (
    CoursePaymentsForm, CoursePaymentsStatusForm, CourseStatsForm,
)
from ...models.agegroup import AgeGroup
from ...models.citizenship import Citizenship
from ...models.courses import Course
from ...models.roles import Participant
from ...models.subjects import (
    SubjectPayment, SubjectRegistrationParticipant, SubjectType,
)
from . import ReportBaseView


class ReportCoursePaymentsView(ReportBaseView):
    form_class = CoursePaymentsForm
    template_name = 'leprikon/reports/course_payments.html'
    title = _('Course payments')
    submit_label = _('Show')
    back_url = reverse('leprikon:report_list')

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['payments'] = SubjectPayment.objects.filter(
            registration__subject__subject_type__subject_type=SubjectType.COURSE,
            accounted__gte=context['date_start'],
            accounted__lte=context['date_end'],
        )
        context['sum'] = context['payments'].aggregate(Sum('amount'))['amount__sum']
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))


class ReportCoursePaymentsStatusView(ReportBaseView):
    form_class = CoursePaymentsStatusForm
    template_name = 'leprikon/reports/course_payments_status.html'
    title = _('Course payments status')
    submit_label = _('Show')
    back_url = reverse('leprikon:report_list')

    CoursePaymentsStatusSums = namedtuple('CoursePaymentsStatusSums', ('registrations', 'status'))

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['reports'] = [
            self.Report(course, context['date'])
            for course in Course.objects.filter(school_year=self.request.school_year)
        ]
        context['sum'] = self.CoursePaymentsStatusSums(
            registrations=sum(len(r.registrations) for r in context['reports']),
            status=sum(r.status for r in context['reports']),
        )
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))

    class Report:
        def __init__(self, course, d):
            self.course = course
            self.date = d

        @cached_property
        def registrations(self):
            return list(self.course.registrations.filter(
                approved__lte=self.date,
            ))

        RegPaymentStatus = namedtuple('RegPaymentStatus', ('registration', 'status'))

        @cached_property
        def registration_statuses(self):
            return [
                registration_status for registration_status in (
                    self.RegPaymentStatus(
                        registration=registration,
                        status=registration.courseregistration.get_payment_statuses(self.date).total,
                    )
                    for registration in self.registrations
                ) if registration_status.status.receivable
            ]

        @cached_property
        def status(self):
            return sum(rs.status for rs in self.registration_statuses)


class ReportCourseStatsView(ReportBaseView):
    form_class = CourseStatsForm
    template_name = 'leprikon/reports/course_stats.html'
    title = _('Course statistics')
    submit_label = _('Show')
    back_url = reverse('leprikon:report_list')

    ReportItem = namedtuple('ReportItem', ('age_group', 'all', 'boys', 'girls', 'citizenships'))

    def form_valid(self, form):
        d = form.cleaned_data['date']
        paid_only = form.cleaned_data['paid_only']
        context = form.cleaned_data
        context['form'] = form

        courses = Course.objects.filter(
            school_year_division__periods__start__lte=d,
            school_year_division__periods__end__gte=d,
        ).distinct()
        context['courses_count'] = courses.count()

        participants = SubjectRegistrationParticipant.objects.filter(
            registration__subject__in=courses,
            registration__approved__date__lte=d,
        ).exclude(registration__canceled__date__lte=d).select_related('registration')
        if paid_only:
            participants = [
                participant for participant in participants
                if participant.registration.courseregistration.get_payment_statuses(d).partial.balance >= 0
            ]
        else:
            participants = list(participants)

        citizenships = list(Citizenship.objects.all())
        context['citizenships'] = citizenships

        context['participants_counts'] = self.ReportItem(
            age_group=None,
            all=len(participants),
            boys=len([p for p in participants if p.gender == Participant.MALE]),
            girls=len([p for p in participants if p.gender == Participant.FEMALE]),
            citizenships=[
                len([p for p in participants if p.citizenship_id == citizenship.id])
                for citizenship in citizenships
            ]
        )
        context['participants_counts_by_age_groups'] = []
        for age_group in AgeGroup.objects.all():
            parts = [p for p in participants if p.age_group == age_group]
            context['participants_counts_by_age_groups'].append(self.ReportItem(
                age_group=age_group,
                all=len(parts),
                boys=len([p for p in parts if p.gender == Participant.MALE]),
                girls=len([p for p in parts if p.gender == Participant.FEMALE]),
                citizenships=[
                    len([p for p in parts if p.citizenship_id == citizenship.id])
                    for citizenship in citizenships
                ]
            ))

        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



import uuid

from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse_lazy as reverse
from django.db import models
from django.template.loader import get_template
from django.utils.encoding import force_text
from django.utils.functional import cached_property
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField
from filer.fields.file import FilerFileField

from ..conf import settings
from .leprikonsite import LeprikonSite


class Message(models.Model):
    created = models.DateTimeField(_('created'), editable=False, auto_now_add=True)
    subject = models.CharField(_('subject'), max_length=150)
    text = HTMLField(_('text'), default='')

    class Meta:
        app_label = 'leprikon'
        ordering = ('-created',)
        verbose_name = _('message')
        verbose_name_plural = _('messages')

    def __str__(self):
        return '{}'.format(self.subject)

    @cached_property
    def all_attachments(self):
        return list(self.attachments.all())

    @cached_property
    def all_recipients(self):
        return list(self.recipients.all())


class MessageRecipient(models.Model):
    slug = models.SlugField(editable=False)
    message = models.ForeignKey(Message, verbose_name=_('message'), related_name='recipients', editable=False)
    recipient = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('recipient'), editable=False,
                                  related_name='leprikon_messages')
    sent = models.DateTimeField(_('sent'), editable=False, auto_now_add=True)
    viewed = models.DateTimeField(_('viewed on site'), editable=False, null=True, default=None)
    sent_mail = models.DateTimeField(_('sent by email'), editable=False, null=True, default=None)

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('recipient')
        verbose_name_plural = _('recipients')
        ordering = ('-sent',)
        unique_together = (('message', 'recipient'),)

    def __str__(self):
        return '{} ({})'.format(self.recipient.get_username(), self.recipient.get_full_name())

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = uuid.uuid4()
            while MessageRecipient.objects.filter(slug=self.slug).exists():
                self.slug = uuid.uuid4()
        super(MessageRecipient, self).save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('leprikon:message_detail', args=(self.slug,))

    def send_mail(self):
        if self.recipient.email:
            context = {
                'message_recipient': self,
                'site': LeprikonSite.objects.get_current(),
            }
            msg = EmailMultiAlternatives(
                subject=self.message.subject,
                body=get_template('leprikon/message_mail.txt').render(context),
                from_email=settings.SERVER_EMAIL,
                to=[self.recipient.email],
                headers={'X-Mailer': 'Leprikon (http://leprikon.cz/)'},
            )
            msg.attach_alternative(get_template('leprikon/message_mail.html').render(context), 'text/html')
            for attachment in self.message.attachments.all():
                msg.attach_file(attachment.file.file.path)
            msg.send()
            self.sent_mail = now()
            self.save()


class MessageAttachment(models.Model):
    message = models.ForeignKey(Message, verbose_name=_('message'), related_name='attachments')
    file = FilerFileField(related_name='+')

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('attachment')
        verbose_name_plural = _('attachments')

    def __str__(self):
        return force_text(self.file)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-09-07 16:18
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields


def create_subject_registration_participants(apps, schema_editor):
    SubjectRegistration = apps.get_model('leprikon', 'SubjectRegistration')
    SubjectRegistrationParticipant = apps.get_model('leprikon', 'SubjectRegistrationParticipant')
    SubjectRegistrationParticipant.objects.bulk_create(
        SubjectRegistrationParticipant(
            registration=reg,
            participant_gender=reg.participant_gender,
            participant_first_name=reg.participant_first_name,
            participant_last_name=reg.participant_last_name,
            participant_birth_num=reg.participant_birth_num,
            participant_street=reg.participant_street,
            participant_city=reg.participant_city,
            participant_postal_code=reg.participant_postal_code,
            participant_phone=reg.participant_phone,
            participant_email=reg.participant_email,
            participant_school_other=reg.participant_school_other,
            participant_school_class=reg.participant_school_class,
            participant_health=reg.participant_health,
            has_parent1=reg.has_parent1,
            parent1_first_name=reg.parent1_first_name,
            parent1_last_name=reg.parent1_last_name,
            parent1_street=reg.parent1_street,
            parent1_city=reg.parent1_city,
            parent1_postal_code=reg.parent1_postal_code,
            parent1_phone=reg.parent1_phone,
            parent1_email=reg.parent1_email,
            has_parent2=reg.has_parent2,
            parent2_first_name=reg.parent2_first_name,
            parent2_last_name=reg.parent2_last_name,
            parent2_street=reg.parent2_street,
            parent2_city=reg.parent2_city,
            parent2_postal_code=reg.parent2_postal_code,
            parent2_phone=reg.parent2_phone,
            parent2_email=reg.parent2_email,
            answers=reg.answers,
            participant_age_group_id=reg.participant_age_group_id,
            participant_citizenship_id=reg.participant_citizenship_id,
            participant_school_id=reg.participant_school_id,
        )
        for reg in SubjectRegistration.objects.iterator()
    )


def create_journalentry_participants(apps, schema_editor):
    JournalEntry = apps.get_model('leprikon', 'JournalEntry')
    SubjectRegistrationParticipant = apps.get_model('leprikon', 'SubjectRegistrationParticipant')
    for journal_entry in JournalEntry.objects.iterator():
        journal_entry.participants.set(SubjectRegistrationParticipant.objects.filter(
            registration__journal_entries=journal_entry,
        ))


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0031_registration_questions_agreements'),
    ]

    operations = [
        migrations.CreateModel(
            name='SubjectRegistrationGroupMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(max_length=30, verbose_name='last name')),
                ('note', models.CharField(blank=True, default='', max_length=150, verbose_name='note')),
            ],
            options={
                'verbose_name': 'group member',
                'verbose_name_plural': 'group members',
            },
        ),
        migrations.CreateModel(
            name='SubjectRegistrationParticipant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('participant_gender', models.CharField(choices=[('m', 'male'), ('f', 'female')], editable=False, max_length=1, verbose_name='gender')),
                ('participant_first_name', models.CharField(max_length=30, verbose_name='first name')),
                ('participant_last_name', models.CharField(max_length=30, verbose_name='last name')),
                ('participant_birth_num', leprikon.models.fields.BirthNumberField(verbose_name='birth number')),
                ('participant_street', models.CharField(max_length=150, verbose_name='street')),
                ('participant_city', models.CharField(max_length=150, verbose_name='city')),
                ('participant_postal_code', leprikon.models.fields.PostalCodeField(verbose_name='postal code')),
                ('participant_phone', models.CharField(blank=True, default='', max_length=30, verbose_name='phone')),
                ('participant_email', leprikon.models.fields.EmailField(blank=True, default='', max_length=254, verbose_name='email address')),
                ('participant_school_other', models.CharField(blank=True, default='', max_length=150, verbose_name='other school')),
                ('participant_school_class', models.CharField(blank=True, default='', max_length=30, verbose_name='class')),
                ('participant_health', models.TextField(blank=True, default='', verbose_name='health')),
                ('has_parent1', models.BooleanField(default=False, verbose_name='first parent')),
                ('parent1_first_name', models.CharField(blank=True, max_length=30, null=True, verbose_name='first name')),
                ('parent1_last_name', models.CharField(blank=True, max_length=30, null=True, verbose_name='last name')),
                ('parent1_street', models.CharField(blank=True, max_length=150, null=True, verbose_name='street')),
                ('parent1_city', models.CharField(blank=True, max_length=150, null=True, verbose_name='city')),
                ('parent1_postal_code', leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name='postal code')),
                ('parent1_phone', models.CharField(blank=True, max_length=30, null=True, verbose_name='phone')),
                ('parent1_email', leprikon.models.fields.EmailField(blank=True, max_length=254, null=True, verbose_name='email address')),
                ('has_parent2', models.BooleanField(default=False, verbose_name='second parent')),
                ('parent2_first_name', models.CharField(blank=True, max_length=30, null=True, verbose_name='first name')),
                ('parent2_last_name', models.CharField(blank=True, max_length=30, null=True, verbose_name='last name')),
                ('parent2_street', models.CharField(blank=True, max_length=150, null=True, verbose_name='street')),
                ('parent2_city', models.CharField(blank=True, max_length=150, null=True, verbose_name='city')),
                ('parent2_postal_code', leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name='postal code')),
                ('parent2_phone', models.CharField(blank=True, max_length=30, null=True, verbose_name='phone')),
                ('parent2_email', leprikon.models.fields.EmailField(blank=True, max_length=254, null=True, verbose_name='email address')),
                ('answers', models.TextField(blank=True, default='{}', editable=False, verbose_name='additional answers')),
                ('participant_age_group', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.AgeGroup', verbose_name='age group')),
                ('participant_citizenship', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.Citizenship', verbose_name='citizenship')),
                ('participant_school', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.School', verbose_name='school')),
            ],
            options={
                'verbose_name': 'participant',
                'verbose_name_plural': 'participants',
            },
        ),
        migrations.AddField(
            model_name='subjectregistration',
            name='group_name',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='group name'),
        ),
        migrations.AddField(
            model_name='subjectregistrationgroupmember',
            name='registration',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='group_members', to='leprikon.SubjectRegistration', verbose_name='registration'),
        ),
        migrations.AddField(
            model_name='subjectregistrationparticipant',
            name='registration',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='participants', to='leprikon.SubjectRegistration', verbose_name='registration'),
        ),
        migrations.RenameField(
            model_name='subject',
            old_name='max_count',
            new_name='max_registrations_count',
        ),
        migrations.RenameField(
            model_name='subject',
            old_name='min_count',
            new_name='min_registrations_count',
        ),
        migrations.AlterField(
            model_name='subject',
            name='max_registrations_count',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='maximal registrations count'),
        ),
        migrations.AlterField(
            model_name='subject',
            name='min_registrations_count',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='minimal registrations count'),
        ),
        migrations.AddField(
            model_name='subject',
            name='max_group_members_count',
            field=models.PositiveIntegerField(default=0, help_text='Group member details only include name and note.', verbose_name='maximal group members count per registration'),
        ),
        migrations.AddField(
            model_name='subject',
            name='min_group_members_count',
            field=models.PositiveIntegerField(default=0, help_text='Group member details only include name and note.', verbose_name='minimal group members count per registration'),
        ),
        migrations.AddField(
            model_name='subject',
            name='max_participants_count',
            field=models.PositiveIntegerField(default=1, help_text='Participant details include birth number (birth day), age group, contacts, parent, etc.', verbose_name='maximal participants count per registration'),
        ),
        migrations.AddField(
            model_name='subject',
            name='min_participants_count',
            field=models.PositiveIntegerField(default=1, help_text='Participant details include birth number (birth day), age group, contacts, parent, etc.', verbose_name='minimal participants count per registration'),
        ),
        migrations.RunPython(create_subject_registration_participants),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='answers',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='has_parent1',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='has_parent2',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent1_city',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent1_email',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent1_first_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent1_last_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent1_phone',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent1_postal_code',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent1_street',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent2_city',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent2_email',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent2_first_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent2_last_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent2_phone',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent2_postal_code',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='parent2_street',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_age_group',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_birth_num',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_citizenship',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_city',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_email',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_first_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_gender',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_health',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_last_name',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_phone',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_postal_code',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_school',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_school_class',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_school_other',
        ),
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_street',
        ),
        migrations.AddField(
            model_name='journalentry',
            name='participants',
            field=models.ManyToManyField(blank=True, related_name='journal_entries', to='leprikon.SubjectRegistrationParticipant', verbose_name='participants'),
        ),
        migrations.RunPython(create_journalentry_participants),
        migrations.RemoveField(
            model_name='journalentry',
            name='registrations',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-10-04 20:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bankreader', '0001_initial'),
        ('leprikon', '0021_user_agreement'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectpayment',
            name='bankreader_transaction',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bankreader.Transaction', verbose_name='bank account transaction'),
        ),
    ]



from time import time

from django.contrib.sites.models import Site
from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django_pays.models import Gateway
from djangocms_text_ckeditor.fields import HTMLField
from localflavor.generic.models import BICField, IBANField

from ..conf import settings
from .account import AccountClosure
from .agreements import Agreement
from .fields import EmailField, PostalCodeField
from .printsetup import PrintSetup
from .utils import BankAccount


class LeprikonSiteManager(models.Manager):
    _cached_site = None
    _cache_timestamp = None

    def get_current(self):
        now = time()
        if not self._cached_site or now - self._cache_timestamp > settings.LEPRIKON_SITE_CACHE_MAX_AGE:
            lookup_kwargs = {}
            if getattr(settings, 'SITE_ID', ''):
                lookup_kwargs['pk'] = settings.SITE_ID
            self._cached_site = self.get_or_create(**lookup_kwargs)[0]
            self._cache_timestamp = now
        return self._cached_site

    def get_queryset(self):
        return super(LeprikonSiteManager, self).get_queryset().select_related(
            'bill_print_setup',
            'reg_print_setup',
        )


class LeprikonSite(Site):
    company_name = models.CharField(_('company name'), max_length=150, blank=True, null=True)
    street = models.CharField(_('street'), max_length=150, blank=True, null=True)
    city = models.CharField(_('city'), max_length=150, blank=True, null=True)
    postal_code = PostalCodeField(_('postal code'), blank=True, null=True)
    email = EmailField(_('email address'), blank=True, null=True)
    phone = models.CharField(_('phone'), max_length=30, blank=True, null=True)
    company_num = models.CharField(_('company number'), max_length=8, blank=True, null=True)
    vat_number = models.CharField(_('VAT number'), max_length=10, blank=True, null=True)
    iban = IBANField(_('IBAN'), blank=True, null=True)
    bic = BICField(_('BIC (SWIFT)'), blank=True, null=True)
    bill_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                         verbose_name=_('bill print setup'), blank=True, null=True)
    reg_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                        verbose_name=_('registration print setup'), blank=True, null=True)
    user_agreement = HTMLField(_('user agreement'), blank=True, default='')
    user_agreement_changed = models.DateTimeField(_('last time user agreement changed'), blank=True, null=True)
    registration_agreements = models.ManyToManyField(
        Agreement, verbose_name=_('registration agreements'), blank=True, related_name='+',
        help_text=_('Add legal agreements for the registration form.'),
    )
    payment_gateway = models.ForeignKey(Gateway, on_delete=models.SET_NULL, related_name='+',
                                        verbose_name=_('Payment Gateway'), blank=True, null=True)

    objects = LeprikonSiteManager()

    class Meta:
        app_label = 'leprikon'
        verbose_name = _('leprikon site')
        verbose_name_plural = _('leprikon sites')

    def get_company_name(self):
        return self.company_name or self.name

    @cached_property
    def bank_account(self):
        return self.iban and BankAccount(self.iban)

    @cached_property
    def max_closure_date(self):
        return AccountClosure.objects.max_closure_date()



from django import forms
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.shortcuts import render
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from ..models.roles import Contact, Leader
from ..models.schoolyear import SchoolYear
from ..models.subjects import SubjectType
from .filters import SchoolYearListFilter
from .messages import SendMessageAdminMixin


class ContactInlineAdmin(admin.TabularInline):
    model = Contact
    extra = 0


@admin.register(Leader)
class LeaderAdmin(SendMessageAdminMixin, admin.ModelAdmin):
    filter_horizontal = ('school_years',)
    inlines = (ContactInlineAdmin,)
    search_fields = ('user__first_name', 'user__last_name', 'contacts__contact')
    list_display = ('id', 'first_name', 'last_name', 'email', 'courses_link',
                    'events_link', 'contacts', 'user_link', 'icon')
    ordering = ('user__first_name', 'user__last_name')
    actions = ('add_school_year',)
    list_filter = (('school_years', SchoolYearListFilter),)
    raw_id_fields = ('user',)

    def add_school_year(self, request, queryset):

        class SchoolYearForm(forms.Form):
            school_year = forms.ModelChoiceField(
                label=_('Target school year'),
                help_text=_('All selected leaders will be added to selected school year.'),
                queryset=SchoolYear.objects.all(),
            )
        if request.POST.get('post', 'no') == 'yes':
            form = SchoolYearForm(request.POST)
            if form.is_valid():
                school_year = form.cleaned_data['school_year']
                for leader in queryset.all():
                    leader.school_years.add(school_year)
                self.message_user(request, _('Selected leaders were added to school year {}.').format(school_year))
                return
        else:
            form = SchoolYearForm()
        return render(request, 'leprikon/admin/action_form.html', {
            'title': _('Select target school year'),
            'queryset': queryset,
            'opts': self.model._meta,
            'form': form,
            'action': 'add_school_year',
            'action_checkbox_name': admin.helpers.ACTION_CHECKBOX_NAME,
        })
    add_school_year.short_description = _('Add selected leaders to another school year')

    def first_name(self, obj):
        return obj.user.first_name
    first_name.short_description = _('first name')
    first_name.admin_order_field = 'user__first_name'

    def last_name(self, obj):
        return obj.user.last_name
    last_name.short_description = _('last name')
    last_name.admin_order_field = 'user__last_name'

    def email(self, obj):
        return obj.user.email
    email.short_description = _('email')
    email.admin_order_field = 'user__email'

    def contacts(self, obj):
        return ', '.join(c.contact for c in obj.all_contacts)
    contacts.short_description = _('contacts')

    def user_link(self, obj):
        return '<a href="{url}">{user}</a>'.format(
            url=reverse('admin:auth_user_change', args=(obj.user.id,)),
            user=obj.user,
        )
    user_link.allow_tags = True
    user_link.short_description = _('user')

    @cached_property
    def courses_url(self):
        return reverse('admin:leprikon_course_changelist')

    @cached_property
    def events_url(self):
        return reverse('admin:leprikon_event_changelist')

    def courses_link(self, obj):
        return '<a href="{url}?leaders__id={leader}">{count}</a>'.format(
            url=self.courses_url,
            leader=obj.id,
            count=obj.subjects.filter(subject_type__subject_type=SubjectType.COURSE).count(),
        )
    courses_link.allow_tags = True
    courses_link.short_description = _('courses')

    def events_link(self, obj):
        return '<a href="{url}?leaders__id={leader}">{count}</a>'.format(
            url=self.events_url,
            leader=obj.id,
            count=obj.subjects.filter(subject_type__subject_type=SubjectType.EVENT).count(),
        )
    events_link.allow_tags = True
    events_link.short_description = _('events')

    def icon(self, obj):
        try:
            return '<img src="{}" alt="{}"/>'.format(obj.photo.icons['48'], obj.photo.label)
        except (AttributeError, KeyError):
            return ''
    icon.allow_tags = True
    icon.short_description = _('photo')

    def get_message_recipients(self, request, queryset):
        return get_user_model().objects.filter(
            leprikon_leader__in=queryset,
        ).distinct()



from django import forms
from django.utils.translation import ugettext_lazy as _

from ..form import GetFormMixin


class EventPaymentsForm(GetFormMixin, forms.Form):
    date_start = forms.DateField(label=_('Start date'))
    date_end = forms.DateField(label=_('End date'))


class EventPaymentsStatusForm(GetFormMixin, forms.Form):
    date = forms.DateField(label=_('Status for date'))


class EventStatsForm(GetFormMixin, forms.Form):
    date = forms.DateField(label=_('Statistics for date'))
    paid_only = forms.BooleanField(label=_('Count only paid registrations'), required=False)



import re
from traceback import print_exc

import requests
from django import template
from django.contrib.staticfiles import finders
from django.core.cache import InvalidCacheBackendError, caches
from django.utils.safestring import mark_safe
from lxml.html import fromstring, tostring

from ..conf import settings
from ..forms.schoolyear import SchoolYearForm
from ..utils import (
    comma_separated as _comma_separated, currency as _currency,
    current_url as _current_url, first_upper as _first_upper,
    url_back as _url_back, url_with_back as _url_with_back,
)

register = template.Library()


@register.filter
def currency(value):
    try:
        return _currency(value)
    except (ValueError, TypeError):
        return ''


@register.filter
def comma_separated(value):
    return _comma_separated(value)


@register.filter
def first_upper(value):
    return _first_upper(value)


@register.filter
def filter_current_school_year(value, school_year):
    return value.filter(school_year=school_year)


@register.filter
def lines(value):
    try:
        return value.strip().split('\n')
    except Exception:
        return []


@register.simple_tag
def param_back():
    return settings.LEPRIKON_PARAM_BACK


@register.simple_tag(takes_context=True)
def url_back(context):
    return _url_back(context['request'])


@register.simple_tag(takes_context=True)
def current_url(context):
    return _current_url(context['request'])


@register.inclusion_tag('leprikon/registration_links.html', takes_context=True)
def registration_links(context, subject):
    context = context.__copy__()
    context['subject'] = subject
    if context['request'].user.is_authenticated():
        context['registrations'] = subject.registrations.filter(user=context['request'].user, canceled=None)
    else:
        context['registrations'] = []
    context['registration_url'] = _url_with_back(subject.get_registration_url(), current_url(context))
    return context


@register.inclusion_tag('leprikon/schoolyear_form.html', takes_context=True)
def school_year_form(context):
    context = context.__copy__()
    context['school_year_form'] = SchoolYearForm(context['request'])
    return context


@register.simple_tag
def upstream(url, xpath, *replacements):
    '''
    If Leprikón is intended to look like another website (the main website of the organization),
    use this tag to include html snippet from the other site to always stay aligned with it.

    Following example includes tag <nav> (with all it's content) from https://example.com:

        {% load cache leprikon_tags %}
        {% cache 300 menu %}{% upstream 'https://example.com/' '//nav' %}{% endcache %}
        {% cache 300 menu %}{% upstream 'https://example.com/' '//nav' '|<br>|<br/>|' %}{% endcache %}
    '''
    try:
        cache = caches['upstream_pages']
    except InvalidCacheBackendError:
        cache = caches['default']
    try:
        content = cache.get(url)
        if content is None:
            content = requests.get(url).content
            cache.set(url, content, 60)
        for replacement in replacements:
            try:
                replacement = replacement.encode('utf-8')
                pattern, repl = replacement[1:-1].split(replacement[:1])
            except (IndexError, ValueError):
                print_exc()
            else:
                content = re.sub(pattern, repl, content)
        return mark_safe(b''.join(
            tostring(node)
            for node in fromstring(content).xpath(xpath)
        ))
    except Exception:
        print_exc()
        return ''


class URLWithBackNode(template.base.Node):
    def __init__(self, original_node):
        self.original_node = original_node

    def render(self, context):
        return _url_with_back(
            self.original_node.render(context),
            current_url(context),
        )


@register.tag
def url_with_back(parser, token):
    """
    Returns an absolute URL as built-in tag url does,
    but adds parameter back with current url.
    """
    return URLWithBackNode(template.defaulttags.url(parser, token))


@register.simple_tag(takes_context=True)
def query_string(context, key, value):
    get = context['request'].GET.copy()
    get[key] = value
    return get.urlencode()


@register.simple_tag
def font(name):
    return finders.find(name)
